"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompletionContext = void 0;
const CompletionError_1 = require("./CompletionError");
class CompletionContext {
    result;
    error;
    constructor(result, error) {
        this.result = result;
        this.error = error;
    }
    setResult(result) {
        this.result = result;
        return this;
    }
    clearResult() {
        this.result = null;
        return this;
    }
    setError(error) {
        this.error = error;
        return this;
    }
    clearError() {
        this.error = null;
        return this;
    }
    getResult() {
        return this.result;
    }
    getError() {
        return this.error;
    }
    hasResult() {
        return this.result != null;
    }
    hasError() {
        return this.error != null;
    }
    combine(other, combiner) {
        if (this.hasError() || other.hasError()) {
            let x = this.getError();
            let y = other.getError();
            if (x != null && y != null)
                return new CompletionContext(combiner(this.getResult(), other.getResult()), new CompletionError_1.CompletionError([x.getCause(), y.getCause()]));
            else if (x != null)
                return new CompletionContext(combiner(this.getResult(), other.getResult()), x);
            else
                return new CompletionContext(combiner(this.getResult(), other.getResult()), y);
        }
        else {
            return CompletionContext.ofResult(combiner(this.getResult(), other.getResult()));
        }
    }
    accept(consumer) {
        consumer(this);
        return this;
    }
    static empty() {
        return new CompletionContext(null, null);
    }
    static ofResult(result) {
        return new CompletionContext(result, null);
    }
    static ofError(error) {
        return new CompletionContext(null, error);
    }
}
exports.CompletionContext = CompletionContext;
//# sourceMappingURL=CompletionContext.js.map